 
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
/**
 * TileGrid for TileGridApplet
 * @author Chris Thiel
 * @version 25 May 2010 3pm
 *
 */

public class TileGrid {
	private int rows;
	private int cols;
	private int x;
	private int y;
	private int size;
	private boolean showLines;
	private ArrayList<Tile> tiles;
	private Tile[][] map;
	private Color bgColor;
	private Color lineColor;

	public TileGrid(int rows, int cols){
		this.rows=rows;
		this.cols=cols;
		this.x=0;
		this.y=0;
		this.size=50;
		this.showLines=true;
		tiles=new ArrayList<Tile>();
		map = new Tile[rows][cols];
		this.bgColor=new Color(244,223,195);
		this.lineColor=new Color(93,66,58);
	}
	public TileGrid(int rows, int cols, boolean s){
		this(rows, cols);
		this.showLines=s;
	}
	public TileGrid(int rows, int cols, int x, int y, int size){
		this(rows, cols);
		this.x=x;
		this.y=y;
		this.size=size;
		this.showLines=true;
	}
	public TileGrid(int rows, int cols, int x, int y, int size,boolean s){
		this(rows,cols);
		this.x=x;
		this.y=y;
		this.size=size;
		this.showLines=s;
	}
	public boolean getShowLines(){
		return showLines;
	}
	public void setShowLines(boolean value){
		this.showLines=value;
	}
	public int getSize(){
		return size;
	}
	public int getNumTiles(){
		return tiles.size();
	}
	public void setSize(int value){
		this.size=value;
	}
	public int getX(){
		return x;
	}
	public int getY(){
		return y;
	}
	public void setX(int x){
		this.x=x;
	}
	public void setY(int y){
		this.y=y;
	}
	public void setLocation(int x, int y){
		this.x=x;
		this.y=y;
	}
	public void setLocation(Point p){
		this.x=p.x;
		this.y=p.y;
	}
	public Point getLocation(){
		return new Point(x,y);
	}
	public void setBGColor(Color c){
		bgColor=c;
	}
	public Color getBGColor(){
		return bgColor;
	}
	/**
	 * Draws grid on Graphics g with top left corner (x,y)
	 * with each cell being size pixels square. It will
	 * update the grid's location and size.
	 * 
	 * @param g the Graphics Object to draw upon
	 * @param x the horizontal location of top left
	 * @param y the vertical location of top left
	 * @param size the size in pixels of each square
	 */
	public void draw(Graphics g, int x, int y, int size){
		this.x=x;
		this.y=y;
		this.size=size;
		Color currentColor=g.getColor();
		g.setColor(bgColor);
		g.fillRect(x, y, cols*size, rows*size);
		for(Tile t:tiles)
			t.draw(g, col2X(t.getCol()), row2Y(t.getRow()),size);
		if (showLines) drawLines(g);
		g.setColor(currentColor);
	}
	private int row2Y(int r) {
		return y+size*r;
	}
	private int col2X(int c) {
		return x+size*c;
	}
	/**
	 * Adds a tile to the grid, telling the tile its new grid, row and column
	 * @param tile
	 * @param row
	 * @param col
	 * @return true if successful, false if occupied or invalid location
	 */
	public boolean add(Tile tile, int row, int col){
		if (!isEmpty(row,col)) return false;
		map[row][col]=tile;
		tiles.add(tile);
		tile.setGrid(this);
		tile.setRow(row);
		tile.setCol(col);
		return true;
	}
	public boolean move(Tile tile, int row, int col){
		if (!isEmpty(row,col)) return false;
		map[row][col]=map[tile.getRow()][tile.getCol()];
		map[tile.getRow()][tile.getCol()]=null;
		tile.setRow(row);
		tile.setCol(col);
		return true;
	}
	/**
	 * Activate tells all the tiles to act
	 * (Usually called after the passage of time).
	 */
	public void activate(){
		int i=0;
		while(i<tiles.size()){
			Tile t=tiles.get(i);
			if (t.getGrid()!=null){//if still alive
				t.act();
				i++;
			}else{
				tiles.remove(t);
			}
			
		}
	}
	/**
	 * Removes a tile from the grid, setting the tile to a null grid and an invalid location
	 * @param tile
	 * @return
	 */
	public boolean remove(Tile tile){
		int r=tile.getRow();
		int c=tile.getCol();
		//if (isEmpty(r,c))return false;
		map[r][c]=null;
		tiles.remove(tile);
		tile.setRow(-1);
		tile.setCol(-1);
		tile.setGrid(null);
		return true;
	}
	/**
	 * checks to see if row r and column c is on the grid
	 * @param r the row to check
	 * @param c the column to check
	 * @return
	 */
	public boolean isValid(int r, int c) {
		
		return r>=0 && r<rows && c>=0 && c<cols;
	}
	/**
	 * checks to see if a tile exists in the
	 * row r and column c
	 * @param r
	 * @param c
	 * @return true if it is an empty cell, false if invalid cell or not empty
	 */
	public boolean isEmpty(int r, int c) {
		if (!isValid(r,c))
			return false;
		return map[r][c]==null;
	}
	/**
	 * Uses the (x,y) location of the screen location
	 * to return a string indicating the (row,col) on the grid
	 * that it corresponds to. 
	 * @param x the horizontal location 
	 * @param y the vertical location
	 * @return a String with the corresponding grid location
	 */
	public String getRowColString(int x, int y){
		String result="(";
		result+=getRow(y)+", ";
		result+=getCol(x);
		return result+")";
	}
	/**
	 * Converts a vertical screen location to a grid row
	 * @param y the vertical location in pixels
	 * @return the row of the click
	 */
	public int getRow(int y){
		if (y<this.y) return -1;
		return (y-this.y)/this.size;
	}
	/**
	 * Converts a horizontal screen location to a grid column
	 * @param x the horizontal location in pixels
	 * @return the column of the click
	 */
	public int getCol(int x){
		if (x<this.x)return -1;
		return (x-this.x)/this.size;
	}
	public Point getRowCol(int x, int y){
		return new Point(getRow(y),getCol(x));
	}
	public int getRowFromPoint(Point p){
		return (p.y-this.y)/this.size;
	}
	public int getColFromPoint(Point p){
		return (p.x-this.x)/this.size;
	}
	
	
	public Tile get(int row, int col){
		return map[row][col];
	}
	
	private void drawLines(Graphics g){
		g.setColor(Color.WHITE);
		for (int r=0; r<=rows;r++)
			g.drawLine(x, y+r*size-2, x+cols*size, y+r*size-2);
		for(int c=0; c<=cols; c++)
			g.drawLine(x+c*size-2, y, x+c*size-2, y+rows*size);
		g.setColor(lineColor);
		for (int r=0; r<=rows;r++)
			g.drawLine(x, y+r*size, x+cols*size, y+r*size);
		for(int c=0; c<=cols; c++)
			g.drawLine(x+c*size, y, x+c*size, y+rows*size);
			
	}
}
